@@ -291,6 +291,7 @@ sub combinations_from_file($$)
     @combs = ({ skip => $skip_combinations{$filename} });
   } else {
     return () if @::opt_combinations or not -f $filename;
+    return () if ::using_extern();
     # Read combinations file in my.cnf format
     mtr_verbose("Read combinations file $filename");
     my $config= My::Config->new($filename);
@@ -1036,7 +1037,7 @@ sub get_tags_from_file($$) {
     }
 
     # Check for a sourced include file.
-    if ($line =~ /^[[:space:]]*(--)?[[:space:]]*source[[:space:]]+([^;[:space:]]+)/)
+    if ($line =~ /^(--)?[[:space:]]*source[[:space:]]+([^;[:space:]]+)/)
     {
       my $include= $2;
       # The rules below must match open_file() function of mysqltest.cc
