@@ -185,7 +185,7 @@ extern "C" sig_handler handle_fatal_signal(int sig)
                           (uint) thread_scheduler->max_threads +
                           (uint) extra_max_connections);
 
-  my_safe_printf_stderr("thread_count=%u\n", (uint) thread_count);
+  my_safe_printf_stderr("thread_count=%u\n", THD_count::value());
 
   if (dflt_key_cache && thread_scheduler)
   {
