@@ -140,7 +140,7 @@ PQRYRES MyColumns(PGLOBAL g, THD *thd, const char *host, const char *db,
 	PCSZ    fmt;
 	char   *fld, *colname, *chset, v, buf[128], uns[16], zero[16];
   int     i, n, nf = 0, ncol = sizeof(buftyp) / sizeof(int);
-  int     len, type, prec, rc, k = 0;
+  int     len, type, prec, rc;
 	bool    b;
   PQRYRES qrp;
   PCOLRES crp;
@@ -399,19 +399,15 @@ PQRYRES SrcColumns(PGLOBAL g, const char *host, const char *db,
   int     w;
   MYSQLC  myc;
   PQRYRES qrp = NULL;
-  const char *p;
 
   if (!port)
     port = mysqld_port;
 
 	if (!strnicmp(srcdef, "select ", 7) || strstr(srcdef, "%s")) {
-		query = (char *)PlugSubAlloc(g, NULL, strlen(srcdef) + 10);
+    query = (char *)PlugSubAlloc(g, NULL, strlen(srcdef) + 10);
 
-		if ((p= strstr(srcdef, "%s")))
-		{
-			/* Replace %s with 1=1 */
-			sprintf(query, "%.*s1=1%s", (int) (p - srcdef), srcdef, p + 2); // dummy where clause
-		}
+		if (strstr(srcdef, "%s"))
+			sprintf(query, srcdef, "1=1");			 // dummy where clause
 		else 
 		  strcpy(query, srcdef);
 
