@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (C) 2013, 2018, MariaDB Corporation.
+Copyright (C) 2013, 2021, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -72,45 +72,24 @@ Updated 14/02/2015
 #include "snappy-c.h"
 #endif
 
-/** Compress a page_compressed page before writing to a data file.
+/** Compress a page for the given compression algorithm.
 @param[in]	buf		page to be compressed
 @param[out]	out_buf		compressed page
-@param[in]	level		compression level
-@param[in]	block_size	file system block size
-@param[in]	encrypted	whether the page will be subsequently encrypted
-@return actual length of compressed page
-@retval	0	if the page was not compressed */
-ulint fil_page_compress(const byte* buf, byte* out_buf, ulint level,
-			ulint block_size, bool encrypted)
+@param[in]	header_len	header length of the page
+@param[in]	comp_algo	compression algorithm
+@param[in]	comp_level	compression level
+@return actual length of compressed page data
+@retval 0 if the page was not compressed */
+static ulint fil_page_compress_low(
+	const byte*	buf,
+	byte*		out_buf,
+	ulint		header_len,
+	ulint		comp_algo,
+	ulint		comp_level)
 {
-	int comp_level = int(level);
-	ulint header_len = FIL_PAGE_DATA + FIL_PAGE_COMPRESSED_SIZE;
-	/* Cache to avoid change during function execution */
-	ulint comp_method = innodb_compression_algorithm;
-
-	if (encrypted) {
-		header_len += FIL_PAGE_COMPRESSION_METHOD_SIZE;
-	}
-
-	/* Let's not compress file space header or
-	extent descriptor */
-	switch (fil_page_get_type(buf)) {
-	case 0:
-	case FIL_PAGE_TYPE_FSP_HDR:
-	case FIL_PAGE_TYPE_XDES:
-	case FIL_PAGE_PAGE_COMPRESSED:
-		return 0;
-	}
-
-	/* If no compression level was provided to this table, use system
-	default level */
-	if (comp_level == 0) {
-		comp_level = page_zip_level;
-	}
-
 	ulint write_size = srv_page_size - header_len;
 
-	switch (comp_method) {
+	switch (comp_algo) {
 	default:
 		ut_ad(!"unknown compression method");
 		/* fall through */
@@ -120,10 +99,9 @@ ulint fil_page_compress(const byte* buf, byte* out_buf, ulint level,
 		{
 			ulong len = uLong(write_size);
 			if (Z_OK == compress2(
-				    out_buf + header_len, &len,
-				    buf, uLong(srv_page_size), comp_level)) {
-				write_size = len;
-				goto success;
+				    out_buf + header_len, &len, buf,
+				    uLong(srv_page_size), int(comp_level))) {
+				return len;
 			}
 		}
 		break;
@@ -141,10 +119,7 @@ ulint fil_page_compress(const byte* buf, byte* out_buf, ulint level,
 			int(srv_page_size), int(write_size));
 # endif
 
-		if (write_size) {
-			goto success;
-		}
-		break;
+		return write_size;
 #endif /* HAVE_LZ4 */
 #ifdef HAVE_LZO
 	case PAGE_LZO_ALGORITHM: {
@@ -155,8 +130,7 @@ ulint fil_page_compress(const byte* buf, byte* out_buf, ulint level,
 			    out_buf + header_len, &len,
 			    out_buf + srv_page_size)
 		    && len <= write_size) {
-			write_size = len;
-			goto success;
+			return len;
 		}
 		break;
 	}
@@ -170,8 +144,7 @@ ulint fil_page_compress(const byte* buf, byte* out_buf, ulint level,
 			    buf, srv_page_size, out_buf + header_len,
 			    &out_pos, write_size)
 		     && out_pos <= write_size) {
-			write_size = out_pos;
-			goto success;
+			return out_pos;
 		}
 		break;
 	}
@@ -187,8 +160,7 @@ ulint fil_page_compress(const byte* buf, byte* out_buf, ulint level,
 				    reinterpret_cast<const char*>(buf)),
 			    unsigned(srv_page_size), 1, 0, 0)
 		    && len <= write_size) {
-			write_size = len;
-			goto success;
+			return len;
 		}
 		break;
 	}
@@ -204,56 +176,172 @@ ulint fil_page_compress(const byte* buf, byte* out_buf, ulint level,
 			    reinterpret_cast<char*>(out_buf) + header_len,
 			    &len)
 		    && len <= write_size) {
-			write_size = len;
-			goto success;
+			return len;
 		}
 		break;
 	}
 #endif /* HAVE_SNAPPY */
 	}
 
-	srv_stats.pages_page_compression_error.inc();
 	return 0;
-success:
+}
+
+/** Compress a page_compressed page for full crc32 format.
+@param[in]	buf		page to be compressed
+@param[out]	out_buf		compressed page
+@param[in]	flags		tablespace flags
+@param[in]	block_size	file system block size
+@return actual length of compressed page
+@retval 0 if the page was not compressed */
+static ulint fil_page_compress_for_full_crc32(
+	const byte*	buf,
+	byte*		out_buf,
+	ulint		flags,
+	ulint		block_size,
+	bool		encrypted)
+{
+	ulint comp_level = fsp_flags_get_page_compression_level(flags);
+
+	if (comp_level == 0) {
+		comp_level = page_zip_level;
+	}
+
+	const ulint header_len = FIL_PAGE_COMP_ALGO;
+
+	ulint write_size = fil_page_compress_low(
+		buf, out_buf, header_len,
+		fil_space_t::get_compression_algo(flags), comp_level);
+
+	if (write_size == 0) {
+fail:
+		srv_stats.pages_page_compression_error.inc();
+		return 0;
+	}
+
+	write_size += header_len;
+	const ulint actual_size = write_size;
+	/* Write the actual length of the data & page type
+	for full crc32 format. */
+	const bool lsb = fil_space_t::full_crc32_page_compressed_len(flags);
+	/* In the MSB, store the rounded-up page size. */
+	write_size = (write_size + lsb + (4 + 255)) & ~255;
+	if (write_size >= srv_page_size) {
+		goto fail;
+	}
+
+	/* Set up the page header */
+	memcpy(out_buf, buf, header_len);
+	out_buf[FIL_PAGE_TYPE] = 1U << (FIL_PAGE_COMPRESS_FCRC32_MARKER - 8);
+	out_buf[FIL_PAGE_TYPE + 1] = byte(write_size >> 8);
+	/* Clean up the buffer for the remaining write_size (except checksum) */
+	memset(out_buf + actual_size, 0, write_size - actual_size - 4);
+	if (lsb) {
+		/* Store the LSB */
+		out_buf[write_size - 5] = byte(actual_size + (1 + 4));
+	}
+
+	if (!block_size) {
+		block_size = 512;
+	}
+
+	ut_ad(write_size);
+	if (write_size & (block_size - 1)) {
+		size_t tmp = write_size;
+		write_size = (write_size + (block_size - 1))
+			& ~(block_size - 1);
+		memset(out_buf + tmp, 0, write_size - tmp);
+	}
+
+	srv_stats.page_compression_saved.add(srv_page_size - write_size);
+	srv_stats.pages_page_compressed.inc();
+
+	return write_size;
+}
+
+/** Compress a page_compressed page for non full crc32 format.
+@param[in]	buf		page to be compressed
+@param[out]	out_buf		compressed page
+@param[in]	flags		tablespace flags
+@param[in]	block_size	file system block size
+@param[in]	encrypted	whether the page will be subsequently encrypted
+@return actual length of compressed page
+@retval        0       if the page was not compressed */
+static ulint fil_page_compress_for_non_full_crc32(
+	const byte*	buf,
+	byte*		out_buf,
+	ulint		flags,
+	ulint		block_size,
+	bool		encrypted)
+{
+	int comp_level = int(fsp_flags_get_page_compression_level(flags));
+	ulint header_len = FIL_PAGE_DATA + FIL_PAGE_COMP_METADATA_LEN;
+	/* Cache to avoid change during function execution */
+	ulint comp_algo = innodb_compression_algorithm;
+
+	if (encrypted) {
+		header_len += FIL_PAGE_ENCRYPT_COMP_ALGO;
+	}
+
+	/* If no compression level was provided to this table, use system
+	default level */
+	if (comp_level == 0) {
+		comp_level = int(page_zip_level);
+	}
+
+	ulint write_size = fil_page_compress_low(
+				buf, out_buf,
+				header_len, comp_algo, comp_level);
+
+	if (write_size == 0) {
+		srv_stats.pages_page_compression_error.inc();
+		return 0;
+	}
+
 	/* Set up the page header */
 	memcpy(out_buf, buf, FIL_PAGE_DATA);
 	/* Set up the checksum */
-	mach_write_to_4(out_buf+FIL_PAGE_SPACE_OR_CHKSUM, BUF_NO_CHECKSUM_MAGIC);
+	mach_write_to_4(out_buf + FIL_PAGE_SPACE_OR_CHKSUM, BUF_NO_CHECKSUM_MAGIC);
 
 	/* Set up the compression algorithm */
-	mach_write_to_8(out_buf+FIL_PAGE_FILE_FLUSH_LSN_OR_KEY_VERSION, comp_method);
+	mach_write_to_8(out_buf + FIL_PAGE_COMP_ALGO, comp_algo);
 
 	if (encrypted) {
 		/* Set up the correct page type */
-		mach_write_to_2(out_buf+FIL_PAGE_TYPE, FIL_PAGE_PAGE_COMPRESSED_ENCRYPTED);
-		mach_write_to_2(out_buf+FIL_PAGE_DATA+FIL_PAGE_COMPRESSED_SIZE, comp_method);
+		mach_write_to_2(out_buf + FIL_PAGE_TYPE,
+				FIL_PAGE_PAGE_COMPRESSED_ENCRYPTED);
+
+		mach_write_to_2(out_buf + FIL_PAGE_DATA
+				+ FIL_PAGE_ENCRYPT_COMP_ALGO, comp_algo);
 	} else {
 		/* Set up the correct page type */
-		mach_write_to_2(out_buf+FIL_PAGE_TYPE, FIL_PAGE_PAGE_COMPRESSED);
+		mach_write_to_2(out_buf + FIL_PAGE_TYPE, FIL_PAGE_PAGE_COMPRESSED);
 	}
 
 	/* Set up the actual payload lenght */
-	mach_write_to_2(out_buf+FIL_PAGE_DATA, write_size);
+	mach_write_to_2(out_buf + FIL_PAGE_DATA + FIL_PAGE_COMP_SIZE,
+			write_size);
+
+	ut_ad(fil_page_is_compressed(out_buf)
+	      || fil_page_is_compressed_encrypted(out_buf));
+
+	ut_ad(mach_read_from_4(out_buf + FIL_PAGE_SPACE_OR_CHKSUM)
+	      == BUF_NO_CHECKSUM_MAGIC);
+
+	ut_ad(mach_read_from_2(out_buf + FIL_PAGE_DATA + FIL_PAGE_COMP_SIZE)
+	      == write_size);
 
 #ifdef UNIV_DEBUG
-	/* Verify */
-	ut_ad(fil_page_is_compressed(out_buf) || fil_page_is_compressed_encrypted(out_buf));
-	ut_ad(mach_read_from_4(out_buf+FIL_PAGE_SPACE_OR_CHKSUM) == BUF_NO_CHECKSUM_MAGIC);
-	ut_ad(mach_read_from_2(out_buf+FIL_PAGE_DATA) == write_size);
-	ut_ad(mach_read_from_8(out_buf+FIL_PAGE_FILE_FLUSH_LSN_OR_KEY_VERSION) == (ulint)comp_method ||
-		mach_read_from_2(out_buf+FIL_PAGE_DATA+FIL_PAGE_COMPRESSED_SIZE) == (ulint)comp_method);
+	bool is_compressed = (mach_read_from_8(out_buf + FIL_PAGE_COMP_ALGO)
+			      == (ulint) comp_algo);
 
-	/* Verify that page can be decompressed */
-	{
-		page_t tmp_buf[UNIV_PAGE_SIZE_MAX];
-		page_t page[UNIV_PAGE_SIZE_MAX];
-		memcpy(page, out_buf, srv_page_size);
-		ut_ad(fil_page_decompress(tmp_buf, page));
-		ut_ad(!buf_page_is_corrupted(false, page, univ_page_size,
-					     NULL));
-	}
+	bool is_encrypted_compressed =
+		(mach_read_from_2(out_buf + FIL_PAGE_DATA
+				  + FIL_PAGE_ENCRYPT_COMP_ALGO)
+		 == (ulint) comp_algo);
 #endif /* UNIV_DEBUG */
 
+	ut_ad(is_compressed || is_encrypted_compressed);
+
 	write_size+=header_len;
 
 	if (block_size <= 0) {
@@ -265,7 +353,8 @@ success:
 	/* Actual write needs to be alligned on block size */
 	if (write_size % block_size) {
 		size_t tmp = write_size;
-		write_size =  (size_t)ut_uint64_align_up((ib_uint64_t)write_size, block_size);
+		write_size =  (size_t)ut_uint64_align_up(
+				(ib_uint64_t)write_size, block_size);
 		/* Clean up the end of buffer */
 		memset(out_buf+tmp, 0, write_size - tmp);
 #ifdef UNIV_DEBUG
@@ -277,37 +366,206 @@ success:
 	srv_stats.page_compression_saved.add(srv_page_size - write_size);
 	srv_stats.pages_page_compressed.inc();
 
-	/* If we do not persistently trim rest of page, we need to write it
-	all */
-	if (!srv_use_trim) {
-		memset(out_buf + write_size, 0, srv_page_size - write_size);
+	return write_size;
+}
+
+/** Compress a page_compressed page before writing to a data file.
+@param[in]	buf		page to be compressed
+@param[out]	out_buf		compressed page
+@param[in]	flags		tablespace flags
+@param[in]	block_size	file system block size
+@param[in]	encrypted	whether the page will be subsequently encrypted
+@return actual length of compressed page
+@retval	0	if the page was not compressed */
+ulint fil_page_compress(
+	const byte*	buf,
+	byte*		out_buf,
+	ulint		flags,
+	ulint		block_size,
+	bool		encrypted)
+{
+	/* The full_crc32 page_compressed format assumes this. */
+	ut_ad(!(block_size & 255));
+	ut_ad(ut_is_2pow(block_size));
+
+	/* Let's not compress file space header or
+	extent descriptor */
+	switch (fil_page_get_type(buf)) {
+	case 0:
+	case FIL_PAGE_TYPE_FSP_HDR:
+	case FIL_PAGE_TYPE_XDES:
+	case FIL_PAGE_PAGE_COMPRESSED:
+		return 0;
 	}
 
-	return write_size;
+	if (fil_space_t::full_crc32(flags)) {
+		return fil_page_compress_for_full_crc32(
+				buf, out_buf, flags, block_size, encrypted);
+	}
+
+	return fil_page_compress_for_non_full_crc32(
+			buf, out_buf, flags, block_size, encrypted);
 }
 
 /** Decompress a page that may be subject to page_compressed compression.
 @param[in,out]	tmp_buf		temporary buffer (of innodb_page_size)
 @param[in,out]	buf		possibly compressed page buffer
+@param[in]	comp_algo	compression algorithm
+@param[in]	header_len	header length of the page
+@param[in]	actual size	actual size of the page
+@retval true if the page is decompressed or false */
+static bool fil_page_decompress_low(
+	byte*		tmp_buf,
+	byte*		buf,
+	ulint		comp_algo,
+	ulint		header_len,
+	ulint		actual_size)
+{
+	switch (comp_algo) {
+	default:
+		ib::error() << "Unknown compression algorithm "
+			    << comp_algo;
+		return false;
+	case PAGE_ZLIB_ALGORITHM:
+		{
+			uLong len = srv_page_size;
+			return (Z_OK == uncompress(tmp_buf, &len,
+					       buf + header_len,
+					       uLong(actual_size))
+				&& len == srv_page_size);
+		}
+#ifdef HAVE_LZ4
+	case PAGE_LZ4_ALGORITHM:
+		return LZ4_decompress_safe(
+			reinterpret_cast<const char*>(buf) + header_len,
+			reinterpret_cast<char*>(tmp_buf),
+			actual_size, srv_page_size) == int(srv_page_size);
+#endif /* HAVE_LZ4 */
+#ifdef HAVE_LZO
+	case PAGE_LZO_ALGORITHM:
+		{
+			lzo_uint len_lzo = srv_page_size;
+			return (LZO_E_OK == lzo1x_decompress_safe(
+					buf + header_len,
+					actual_size, tmp_buf, &len_lzo, NULL)
+				&& len_lzo == srv_page_size);
+		}
+#endif /* HAVE_LZO */
+#ifdef HAVE_LZMA
+	case PAGE_LZMA_ALGORITHM:
+		{
+			size_t		src_pos = 0;
+			size_t		dst_pos = 0;
+			uint64_t 	memlimit = UINT64_MAX;
+
+			return LZMA_OK == lzma_stream_buffer_decode(
+				&memlimit, 0, NULL, buf + header_len,
+				&src_pos, actual_size, tmp_buf, &dst_pos,
+				srv_page_size)
+				&& dst_pos == srv_page_size;
+		}
+#endif /* HAVE_LZMA */
+#ifdef HAVE_BZIP2
+	case PAGE_BZIP2_ALGORITHM:
+		{
+			unsigned int dst_pos = srv_page_size;
+			return BZ_OK == BZ2_bzBuffToBuffDecompress(
+				reinterpret_cast<char*>(tmp_buf),
+				&dst_pos,
+				reinterpret_cast<char*>(buf) + header_len,
+				actual_size, 1, 0)
+				&& dst_pos == srv_page_size;
+		}
+#endif /* HAVE_BZIP2 */
+#ifdef HAVE_SNAPPY
+	case PAGE_SNAPPY_ALGORITHM:
+		{
+			size_t olen = srv_page_size;
+
+			return SNAPPY_OK == snappy_uncompress(
+				reinterpret_cast<const char*>(buf)
+				+ header_len,
+				actual_size,
+				reinterpret_cast<char*>(tmp_buf), &olen)
+				&& olen == srv_page_size;
+		}
+#endif /* HAVE_SNAPPY */
+	}
+
+	return false;
+}
+
+/** Decompress a page for full crc32 format.
+@param[in,out]	tmp_buf	temporary buffer (of innodb_page_size)
+@param[in,out]	buf	possibly compressed page buffer
+@param[in]	flags	tablespace flags
 @return size of the compressed data
 @retval	0		if decompression failed
 @retval	srv_page_size	if the page was not compressed */
-ulint fil_page_decompress(byte* tmp_buf, byte* buf)
+ulint fil_page_decompress_for_full_crc32(byte* tmp_buf, byte* buf, ulint flags)
+{
+	ut_ad(fil_space_t::full_crc32(flags));
+	bool compressed = false;
+	size_t size = buf_page_full_crc32_size(buf, &compressed, NULL);
+	if (!compressed) {
+		ut_ad(size == srv_page_size);
+		return size;
+	}
+
+	if (!fil_space_t::is_compressed(flags)) {
+		return 0;
+	}
+
+	if (size >= srv_page_size) {
+		return 0;
+	}
+
+	if (fil_space_t::full_crc32_page_compressed_len(flags)) {
+		compile_time_assert(FIL_PAGE_FCRC32_CHECKSUM == 4);
+		if (size_t lsb = buf[size - 5]) {
+			size += lsb - 0x100;
+		}
+		size -= 5;
+	}
+
+	const size_t header_len = FIL_PAGE_COMP_ALGO;
+
+	if (!fil_page_decompress_low(tmp_buf, buf,
+				     fil_space_t::get_compression_algo(flags),
+				     header_len, size - header_len)) {
+		return 0;
+	}
+
+	srv_stats.pages_page_decompressed.inc();
+	memcpy(buf, tmp_buf, srv_page_size);
+	return size;
+}
+
+/** Decompress a page for non full crc32 format.
+@param[in,out] tmp_buf	temporary buffer (of innodb_page_size)
+@param[in,out] buf	possibly compressed page buffer
+@return size of the compressed data
+@retval	0		if decompression failed
+@retval	srv_page_size	if the page was not compressed */
+ulint fil_page_decompress_for_non_full_crc32(
+	byte*	tmp_buf,
+	byte*	buf)
 {
 	const unsigned	ptype = mach_read_from_2(buf+FIL_PAGE_TYPE);
 	ulint header_len;
-	uint64_t compression_alg;
+	uint comp_algo;
 	switch (ptype) {
 	case FIL_PAGE_PAGE_COMPRESSED_ENCRYPTED:
-		header_len = FIL_PAGE_DATA + FIL_PAGE_COMPRESSED_SIZE
-			+ FIL_PAGE_COMPRESSION_METHOD_SIZE;
-		compression_alg = mach_read_from_2(
-			FIL_PAGE_DATA + FIL_PAGE_COMPRESSED_SIZE + buf);
+		header_len= FIL_PAGE_DATA + FIL_PAGE_ENCRYPT_COMP_METADATA_LEN;
+		comp_algo = mach_read_from_2(
+			FIL_PAGE_DATA + FIL_PAGE_ENCRYPT_COMP_ALGO + buf);
 		break;
 	case FIL_PAGE_PAGE_COMPRESSED:
-		header_len = FIL_PAGE_DATA + FIL_PAGE_COMPRESSED_SIZE;
-		compression_alg = mach_read_from_8(
-			FIL_PAGE_FILE_FLUSH_LSN_OR_KEY_VERSION + buf);
+		header_len = FIL_PAGE_DATA + FIL_PAGE_COMP_METADATA_LEN;
+		if (mach_read_from_6(FIL_PAGE_COMP_ALGO + buf)) {
+			return 0;
+		}
+		comp_algo = mach_read_from_2(FIL_PAGE_COMP_ALGO + 6 + buf);
 		break;
 	default:
 		return srv_page_size;
@@ -318,99 +576,39 @@ ulint fil_page_decompress(byte* tmp_buf, byte* buf)
 		return 0;
 	}
 
-	ulint actual_size = mach_read_from_2(buf + FIL_PAGE_DATA);
+	ulint actual_size = mach_read_from_2(buf + FIL_PAGE_DATA
+					     + FIL_PAGE_COMP_SIZE);
 
 	/* Check if payload size is corrupted */
 	if (actual_size == 0 || actual_size > srv_page_size - header_len) {
 		return 0;
 	}
 
-	switch (compression_alg) {
-	default:
-		ib::error() << "Unknown compression algorithm "
-			    << compression_alg;
+	if (!fil_page_decompress_low(tmp_buf, buf, comp_algo, header_len,
+				     actual_size)) {
 		return 0;
-	case PAGE_ZLIB_ALGORITHM:
-		{
-			uLong len = srv_page_size;
-			if (Z_OK == uncompress(tmp_buf, &len,
-					       buf + header_len,
-					       uLong(actual_size))
-			    && len == srv_page_size) {
-				break;
-			}
-		}
-		return 0;
-#ifdef HAVE_LZ4
-	case PAGE_LZ4_ALGORITHM:
-		if (LZ4_decompress_safe(reinterpret_cast<const char*>(buf)
-					+ header_len,
-					reinterpret_cast<char*>(tmp_buf),
-					actual_size, srv_page_size)
-		    == int(srv_page_size)) {
-			break;
-		}
-		return 0;
-#endif /* HAVE_LZ4 */
-#ifdef HAVE_LZO
-	case PAGE_LZO_ALGORITHM: {
-		lzo_uint len_lzo = srv_page_size;
-		if (LZO_E_OK == lzo1x_decompress_safe(
-			    buf + header_len,
-			    actual_size, tmp_buf, &len_lzo, NULL)
-		    && len_lzo == srv_page_size) {
-			break;
-		}
-		return 0;
-        }
-#endif /* HAVE_LZO */
-#ifdef HAVE_LZMA
-	case PAGE_LZMA_ALGORITHM: {
-		size_t		src_pos = 0;
-		size_t		dst_pos = 0;
-		uint64_t 	memlimit = UINT64_MAX;
-
-		if (LZMA_OK == lzma_stream_buffer_decode(
-			    &memlimit, 0, NULL, buf + header_len,
-			    &src_pos, actual_size, tmp_buf, &dst_pos,
-			    srv_page_size)
-		    && dst_pos == srv_page_size) {
-			break;
-		}
-		return 0;
-	}
-#endif /* HAVE_LZMA */
-#ifdef HAVE_BZIP2
-	case PAGE_BZIP2_ALGORITHM: {
-		unsigned int dst_pos = srv_page_size;
-		if (BZ_OK == BZ2_bzBuffToBuffDecompress(
-			    reinterpret_cast<char*>(tmp_buf),
-			    &dst_pos,
-			    reinterpret_cast<char*>(buf) + header_len,
-			    actual_size, 1, 0)
-		    && dst_pos == srv_page_size) {
-			break;
-		}
-		return 0;
-	}
-#endif /* HAVE_BZIP2 */
-#ifdef HAVE_SNAPPY
-	case PAGE_SNAPPY_ALGORITHM: {
-		size_t olen = srv_page_size;
-
-		if (SNAPPY_OK == snappy_uncompress(
-			    reinterpret_cast<const char*>(buf) + header_len,
-			    actual_size,
-			    reinterpret_cast<char*>(tmp_buf), &olen)
-		    && olen == srv_page_size) {
-			break;
-		}
-		return 0;
-	}
-#endif /* HAVE_SNAPPY */
 	}
 
 	srv_stats.pages_page_decompressed.inc();
 	memcpy(buf, tmp_buf, srv_page_size);
 	return actual_size;
 }
+
+/** Decompress a page that may be subject to page_compressed compression.
+@param[in,out]	tmp_buf		temporary buffer (of innodb_page_size)
+@param[in,out]	buf		possibly compressed page buffer
+@param[in]	flags		tablespace flags
+@return size of the compressed data
+@retval	0		if decompression failed
+@retval	srv_page_size	if the page was not compressed */
+ulint fil_page_decompress(
+	byte*	tmp_buf,
+	byte*	buf,
+	ulint	flags)
+{
+	if (fil_space_t::full_crc32(flags)) {
+		return fil_page_decompress_for_full_crc32(tmp_buf, buf, flags);
+	}
+
+	return fil_page_decompress_for_non_full_crc32(tmp_buf, buf);
+}
