@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2007, 2018, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2016, 2020, MariaDB Corporation.
+Copyright (c) 2016, 2021, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -36,6 +36,7 @@ Completed 2011/7/10 Sunny and Jimmy Yang
 #include "ut0list.h"
 #include "zlib.h"
 #include "fts0opt.h"
+#include "fts0vlc.h"
 
 /** The FTS optimize thread's work queue. */
 ib_wqueue_t* fts_optimize_wq;
@@ -492,7 +493,7 @@ fts_index_fetch_nodes(
 
 		fts_get_table_name(fts_table, table_name);
 
-		pars_info_bind_id(info, true, "table_name", table_name);
+		pars_info_bind_id(info, "table_name", table_name);
 	}
 
 	pars_info_bind_function(info, "my_func", fetch->read_record, fetch);
@@ -821,7 +822,7 @@ fts_index_fetch_words(
 			info, "word", word->f_str, word->f_len);
 
 		fts_get_table_name(&optim->fts_index_table, table_name);
-		pars_info_bind_id(info, true, "table_name", table_name);
+		pars_info_bind_id(info, "table_name", table_name);
 
 		graph = fts_parse_sql(
 			&optim->fts_index_table,
@@ -977,7 +978,7 @@ fts_table_fetch_doc_ids(
 	pars_info_bind_function(info, "my_func", fts_fetch_doc_ids, doc_ids);
 
 	fts_get_table_name(fts_table, table_name);
-	pars_info_bind_id(info, true, "table_name", table_name);
+	pars_info_bind_id(info, "table_name", table_name);
 
 	graph = fts_parse_sql(
 		fts_table,
@@ -1116,7 +1117,7 @@ fts_optimize_encode_node(
 	ulint		pos_enc_len;
 	doc_id_t	doc_id_delta;
 	dberr_t		error = DB_SUCCESS;
-	byte*		src = enc->src_ilist_ptr;
+	const byte*	src = enc->src_ilist_ptr;
 
 	if (node->first_doc_id == 0) {
 		ut_a(node->last_doc_id == 0);
@@ -1173,7 +1174,7 @@ fts_optimize_encode_node(
 
 	/* Encode the doc id. Cast to ulint, the delta should be small and
 	therefore no loss of precision. */
-	dst += fts_encode_int((ulint) doc_id_delta, dst);
+	dst = fts_encode_int(doc_id_delta, dst);
 
 	/* Copy the encoded pos array. */
 	memcpy(dst, src, pos_enc_len);
@@ -1220,7 +1221,8 @@ fts_optimize_node(
 		doc_id_t	delta;
 		doc_id_t	del_doc_id = FTS_NULL_DOC_ID;
 
-		delta = fts_decode_vlc(&enc->src_ilist_ptr);
+		delta = fts_decode_vlc(
+			(const byte**)&enc->src_ilist_ptr);
 
 test_again:
 		/* Check whether the doc id is in the delete list, if
@@ -1248,7 +1250,7 @@ test_again:
 
 			/* Skip the entries for this document. */
 			while (*enc->src_ilist_ptr) {
-				fts_decode_vlc(&enc->src_ilist_ptr);
+				fts_decode_vlc((const byte**)&enc->src_ilist_ptr);
 			}
 
 			/* Skip the end of word position marker. */
@@ -1441,7 +1443,7 @@ fts_optimize_write_word(
 
 	fts_table->suffix = fts_get_suffix(selected);
 	fts_get_table_name(fts_table, table_name);
-	pars_info_bind_id(info, true, "table_name", table_name);
+	pars_info_bind_id(info, "table_name", table_name);
 
 	graph = fts_parse_sql(
 		fts_table,
@@ -2033,11 +2035,11 @@ fts_optimize_purge_deleted_doc_ids(
 	used in the fts_delete_doc_ids_sql */
 	optim->fts_common_table.suffix = fts_common_tables[3];
 	fts_get_table_name(&optim->fts_common_table, deleted);
-	pars_info_bind_id(info, true, fts_common_tables[3], deleted);
+	pars_info_bind_id(info, fts_common_tables[3], deleted);
 
 	optim->fts_common_table.suffix = fts_common_tables[4];
 	fts_get_table_name(&optim->fts_common_table, deleted_cache);
-	pars_info_bind_id(info, true, fts_common_tables[4], deleted_cache);
+	pars_info_bind_id(info, fts_common_tables[4], deleted_cache);
 
 	graph = fts_parse_sql(NULL, info, fts_delete_doc_ids_sql);
 
@@ -2090,12 +2092,11 @@ fts_optimize_purge_deleted_doc_id_snapshot(
 	used in the fts_end_delete_sql */
 	optim->fts_common_table.suffix = fts_common_tables[0];
 	fts_get_table_name(&optim->fts_common_table, being_deleted);
-	pars_info_bind_id(info, true, fts_common_tables[0], being_deleted);
+	pars_info_bind_id(info, fts_common_tables[0], being_deleted);
 
 	optim->fts_common_table.suffix = fts_common_tables[1];
 	fts_get_table_name(&optim->fts_common_table, being_deleted_cache);
-	pars_info_bind_id(info, true, fts_common_tables[1],
-			  being_deleted_cache);
+	pars_info_bind_id(info, fts_common_tables[1], being_deleted_cache);
 
 	/* Delete the doc ids that were copied to delete pending state at
 	the start of optimize. */
@@ -2151,20 +2152,19 @@ fts_optimize_create_deleted_doc_id_snapshot(
 	used in the fts_init_delete_sql */
 	optim->fts_common_table.suffix = fts_common_tables[0];
 	fts_get_table_name(&optim->fts_common_table, being_deleted);
-	pars_info_bind_id(info, true, fts_common_tables[0], being_deleted);
+	pars_info_bind_id(info, fts_common_tables[0], being_deleted);
 
 	optim->fts_common_table.suffix = fts_common_tables[3];
 	fts_get_table_name(&optim->fts_common_table, deleted);
-	pars_info_bind_id(info, true, fts_common_tables[3], deleted);
+	pars_info_bind_id(info, fts_common_tables[3], deleted);
 
 	optim->fts_common_table.suffix = fts_common_tables[1];
 	fts_get_table_name(&optim->fts_common_table, being_deleted_cache);
-	pars_info_bind_id(info, true, fts_common_tables[1],
-			  being_deleted_cache);
+	pars_info_bind_id(info, fts_common_tables[1], being_deleted_cache);
 
 	optim->fts_common_table.suffix = fts_common_tables[4];
 	fts_get_table_name(&optim->fts_common_table, deleted_cache);
-	pars_info_bind_id(info, true, fts_common_tables[4], deleted_cache);
+	pars_info_bind_id(info, fts_common_tables[4], deleted_cache);
 
 	/* Move doc_ids that are to be deleted to state being deleted. */
 	graph = fts_parse_sql(NULL, info, fts_init_delete_sql);
@@ -2642,6 +2642,9 @@ fts_optimize_request_sync_table(
 
 	ib_wqueue_add(fts_optimize_wq, msg, msg->heap, true);
 
+	DBUG_EXECUTE_IF("fts_optimize_wq_count_check",
+			DBUG_ASSERT(fts_optimize_wq->length <= 1000););
+
 	mutex_exit(&fts_optimize_wq->mutex);
 }
 
