@@ -1978,7 +1978,7 @@ static MYSQL_THDVAR_INT(
   "Remote server transmission existence when UDF is used at condition and \"engine_condition_pushdown=1\"", /* comment */
   NULL, /* check */
   NULL, /* update */
-  -1, /* def */
+  0, /* def */
   -1, /* min */
   1, /* max */
   0 /* blk */
@@ -3355,6 +3355,76 @@ int spider_param_slave_trx_isolation()
   DBUG_RETURN(spider_slave_trx_isolation);
 }
 
+/*
+ -1 :not set
+  0-:seconds of timeout
+ */
+static MYSQL_THDVAR_INT(
+  remote_wait_timeout, /* name */
+  PLUGIN_VAR_RQCMDARG, /* opt */
+  "Wait timeout on remote server", /* comment */
+  NULL, /* check */
+  NULL, /* update */
+  -1, /* def */
+  -1, /* min */
+  2147483647, /* max */
+  0 /* blk */
+);
+
+int spider_param_remote_wait_timeout(
+  THD *thd
+) {
+  DBUG_ENTER("spider_param_remote_wait_timeout");
+  if (likely(thd))
+    DBUG_RETURN(THDVAR(thd, remote_wait_timeout));
+  DBUG_RETURN(-1);
+}
+
+/*
+ -1 :not set
+  0-:seconds of timeout
+ */
+static MYSQL_THDVAR_INT(
+  wait_timeout, /* name */
+  PLUGIN_VAR_RQCMDARG, /* opt */
+  "Wait timeout of setting to remote server", /* comment */
+  NULL, /* check */
+  NULL, /* update */
+  604800, /* def */
+  -1, /* min */
+  2147483647, /* max */
+  0 /* blk */
+);
+
+int spider_param_wait_timeout(
+  THD *thd
+) {
+  DBUG_ENTER("spider_param_wait_timeout");
+  if (likely(thd))
+    DBUG_RETURN(THDVAR(thd, wait_timeout));
+  DBUG_RETURN(604800);
+}
+
+/*
+  FALSE: no sync
+  TRUE:  sync
+ */
+static MYSQL_THDVAR_BOOL(
+  sync_sql_mode, /* name */
+  PLUGIN_VAR_OPCMDARG, /* opt */
+  "Sync sql_mode", /* comment */
+  NULL, /* check */
+  NULL, /* update */
+  TRUE /* def */
+);
+
+bool spider_param_sync_sql_mode(
+  THD *thd
+) {
+  DBUG_ENTER("spider_param_sync_sql_mode");
+  DBUG_RETURN(THDVAR(thd, sync_sql_mode));
+}
+
 static struct st_mysql_storage_engine spider_storage_engine =
 { MYSQL_HANDLERTON_INTERFACE_VERSION };
 
@@ -3505,6 +3575,9 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(table_crd_thread_count),
 #endif
   MYSQL_SYSVAR(slave_trx_isolation),
+  MYSQL_SYSVAR(remote_wait_timeout),
+  MYSQL_SYSVAR(wait_timeout),
+  MYSQL_SYSVAR(sync_sql_mode),
   NULL
 };
 
