@@ -1162,9 +1162,12 @@ static size_t
 my_snprintf_mb2(CHARSET_INFO *cs __attribute__((unused)),
                 char* to, size_t n, const char* fmt, ...)
 {
+  size_t ret;
   va_list args;
   va_start(args,fmt);
-  return my_vsnprintf_mb2(to, n, fmt, args);
+  ret= my_vsnprintf_mb2(to, n, fmt, args);
+  va_end(args);
+  return ret;
 }
 
 
@@ -2391,9 +2394,12 @@ static size_t
 my_snprintf_utf32(CHARSET_INFO *cs __attribute__((unused)),
                   char* to, size_t n, const char* fmt, ...)
 {
+  size_t ret;
   va_list args;
   va_start(args,fmt);
-  return my_vsnprintf_utf32(to, n, fmt, args);
+  ret= my_vsnprintf_utf32(to, n, fmt, args);
+  va_end(args);
+  return ret;
 }
 
 
